@@ -32,12 +32,15 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_CONSOLE_STAT_CALC_H_
-#define _D_CONSOLE_STAT_CALC_H_
+#ifndef D_CONSOLE_STAT_CALC_H
+#define D_CONSOLE_STAT_CALC_H
 
 #include "StatCalc.h"
+
+#include <string>
+#include <functional>
+
 #include "TimerA2.h"
-#include "util.h"
 
 namespace aria2 {
 
@@ -47,48 +50,35 @@ protected:
 public:
   virtual ~SizeFormatter() {}
 
-  std::string operator()(int64_t size) const
-  {
-    return format(size);
-  }
-};
-
-class AbbrevSizeFormatter:public SizeFormatter {
-protected:
-  virtual std::string format(int64_t size) const
-  {
-    return util::abbrevSize(size);
-  }
-};
-
-class PlainSizeFormatter:public SizeFormatter {
-protected:
-  virtual std::string format(int64_t size) const
-  {
-    return util::itos(size);
-  }
+  std::string operator()(int64_t size) const;
 };
 
 class ConsoleStatCalc:public StatCalc
 {
 private:
-  Timer _cp;
+  Timer cp_;
 
-  Timer _lastSummaryNotified;
+  Timer lastSummaryNotified_;
 
-  time_t _summaryInterval;
+  time_t summaryInterval_;
 
-  SharedHandle<SizeFormatter> _sizeFormatter;
+  SharedHandle<SizeFormatter> sizeFormatter_;
+  bool readoutVisibility_;
 public:
   ConsoleStatCalc(time_t summaryInterval, bool humanReadable = true);
 
   virtual ~ConsoleStatCalc() {}
 
   virtual void calculateStat(const DownloadEngine* e);
+
+  void setReadoutVisibility(bool visibility)
+  {
+    readoutVisibility_ = visibility;
+  }
 };
 
 typedef SharedHandle<ConsoleStatCalc> ConsoleStatCalcHandle;
 
 } // namespace aria2
 
-#endif // _D_CONSOLE_STAT_CALC_H_
+#endif // D_CONSOLE_STAT_CALC_H
