@@ -45,6 +45,7 @@
 #include "SegmentMan.h"
 #include "Segment.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "ChecksumCheckIntegrityEntry.h"
 #include "PieceStorage.h"
 #include "CheckIntegrityCommand.h"
@@ -55,14 +56,15 @@
 #include "Socket.h"
 #include "message.h"
 #include "prefs.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "RequestGroupMan.h"
 #include "wallclock.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
 #include "SinkStreamFilter.h"
+#include "FileEntry.h"
+#include "SocketRecvBuffer.h"
 #ifdef ENABLE_MESSAGE_DIGEST
-# include "MessageDigestHelper.h"
+# include "MessageDigest.h"
+# include "message_digest_helper.h"
 #endif // ENABLE_MESSAGE_DIGEST
 #ifdef ENABLE_BITTORRENT
 # include "bittorrent_helper.h"
@@ -70,29 +72,29 @@
 
 namespace aria2 {
 
-static const size_t BUFSIZE = 16*1024;
+namespace {
+const size_t BUFSIZE = 16*1024;
+} // namespace
 
-DownloadCommand::DownloadCommand(cuid_t cuid,
-                                 const SharedHandle<Request>& req,
-                                 const SharedHandle<FileEntry>& fileEntry,
-                                 RequestGroup* requestGroup,
-                                 DownloadEngine* e,
-                                 const SocketHandle& s):
-  AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
-  buf_(new unsigned char[BUFSIZE]),
-  startupIdleTime_(10),
-  lowestDownloadSpeedLimit_(0),
-  pieceHashValidationEnabled_(false)
+DownloadCommand::DownloadCommand
+(cuid_t cuid,
+ const SharedHandle<Request>& req,
+ const SharedHandle<FileEntry>& fileEntry,
+ RequestGroup* requestGroup,
+ DownloadEngine* e,
+ const SocketHandle& s,
+ const SharedHandle<SocketRecvBuffer>& socketRecvBuffer)
+  : AbstractCommand(cuid, req, fileEntry, requestGroup, e, s, socketRecvBuffer),
+    startupIdleTime_(10),
+    lowestDownloadSpeedLimit_(0),
+    pieceHashValidationEnabled_(false)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   {
     if(getOption()->getAsBool(PREF_REALTIME_CHUNK_CHECKSUM)) {
       const std::string& algo = getDownloadContext()->getPieceHashAlgo();
-      if(MessageDigestContext::supports(algo)) {
-        messageDigestContext_.reset(new MessageDigestContext());
-        messageDigestContext_->trySetAlgo(algo);
-        messageDigestContext_->digestInit();
-        
+      if(MessageDigest::supports(algo)) {
+        messageDigest_ = MessageDigest::create(algo);
         pieceHashValidationEnabled_ = true;
       }
     }
@@ -106,12 +108,12 @@ DownloadCommand::DownloadCommand(cuid_t cuid,
   streamFilter_.reset(new SinkStreamFilter(pieceHashValidationEnabled_));
   streamFilter_->init();
   sinkFilterOnly_ = true;
+  checkSocketRecvBuffer();
 }
 
 DownloadCommand::~DownloadCommand() {
   peerStat_->downloadStop();
   getSegmentMan()->updateFastestPeerStat(peerStat_);
-  delete [] buf_;
 }
 
 bool DownloadCommand::executeInternal() {
@@ -126,37 +128,53 @@ bool DownloadCommand::executeInternal() {
   const SharedHandle<DiskAdaptor>& diskAdaptor =
     getPieceStorage()->getDiskAdaptor();
   SharedHandle<Segment> segment = getSegments().front();
-  size_t bufSize;
-  if(sinkFilterOnly_) {
-    if(segment->getLength() > 0 ) {
-      if(static_cast<uint64_t>(segment->getPosition()+segment->getLength()) <=
-         static_cast<uint64_t>(getFileEntry()->getLastOffset())) {
-        bufSize = std::min(segment->getLength()-segment->getWrittenLength(),
-                           BUFSIZE);
+  bool eof = false;
+  if(getSocketRecvBuffer()->bufferEmpty()) {
+    // Only read from socket when buffer is empty.  Imagine that When
+    // segment length is *short* and we are using HTTP pilelining.  We
+    // issued 2 requests in pipeline. When reading first response
+    // header, we may read its response body and 2nd response header
+    // and 2nd response body in buffer if they are small enough to fit
+    // in buffer. And then server may sends EOF.  In this case, we
+    // read data from socket here, we will get EOF and leaves 2nd
+    // response unprocessed.  To prevent this, we don't read from
+    // socket when buffer is not empty.
+    eof = getSocketRecvBuffer()->recv() == 0 &&
+      !getSocket()->wantRead() && !getSocket()->wantWrite();
+  }
+  if(!eof) {
+    size_t bufSize;
+    if(sinkFilterOnly_) {
+      if(segment->getLength() > 0) {
+        if(static_cast<uint64_t>(segment->getPosition()+segment->getLength()) <=
+           static_cast<uint64_t>(getFileEntry()->getLastOffset())) {
+          bufSize = std::min(segment->getLength()-segment->getWrittenLength(),
+                             getSocketRecvBuffer()->getBufferLength());
+        } else {
+          bufSize =
+            std::min
+            (static_cast<size_t>
+             (getFileEntry()->getLastOffset()-segment->getPositionToWrite()),
+             getSocketRecvBuffer()->getBufferLength());
+        }
       } else {
-        bufSize =
-          std::min
-          (static_cast<size_t>
-           (getFileEntry()->getLastOffset()-segment->getPositionToWrite()),
-           BUFSIZE);
+        bufSize = getSocketRecvBuffer()->getBufferLength();
       }
+      streamFilter_->transform(diskAdaptor, segment,
+                               getSocketRecvBuffer()->getBuffer(), bufSize);
     } else {
-      bufSize = BUFSIZE;
+      // It is possible that segment is completed but we have some bytes
+      // of stream to read. For example, chunked encoding has "0"+CRLF
+      // after data. After we read data(at this moment segment is
+      // completed), we need another 3bytes(or more if it has trailers).
+      streamFilter_->transform(diskAdaptor, segment,
+                               getSocketRecvBuffer()->getBuffer(),
+                               getSocketRecvBuffer()->getBufferLength());
+      bufSize = streamFilter_->getBytesProcessed();
     }
-    getSocket()->readData(buf_, bufSize);
-    streamFilter_->transform(diskAdaptor, segment, buf_, bufSize);
-  } else {
-    // It is possible that segment is completed but we have some bytes
-    // of stream to read. For example, chunked encoding has "0"+CRLF
-    // after data. After we read data(at this moment segment is
-    // completed), we need another 3bytes(or more if it has trailers).
-    bufSize = BUFSIZE;
-    getSocket()->peekData(buf_, bufSize);
-    streamFilter_->transform(diskAdaptor, segment, buf_, bufSize);
-    bufSize = streamFilter_->getBytesProcessed();
-    getSocket()->readData(buf_, bufSize);
+    getSocketRecvBuffer()->shiftBuffer(bufSize);
+    peerStat_->updateDownloadLength(bufSize);
   }
-  peerStat_->updateDownloadLength(bufSize);
   getSegmentMan()->updateDownloadSpeedFor(peerStat_);
   bool segmentPartComplete = false;
   // Note that GrowSegment::complete() always returns false.
@@ -164,8 +182,7 @@ bool DownloadCommand::executeInternal() {
     if(segment->complete() ||
        segment->getPositionToWrite() == getFileEntry()->getLastOffset()) {
       segmentPartComplete = true;
-    } else if(segment->getLength() == 0 && bufSize == 0 &&
-              !getSocket()->wantRead() && !getSocket()->wantWrite()) {
+    } else if(segment->getLength() == 0 && eof) {
       segmentPartComplete = true;
     }
   } else {
@@ -188,8 +205,7 @@ bool DownloadCommand::executeInternal() {
     }
   }
 
-  if(!segmentPartComplete && bufSize == 0 &&
-     !getSocket()->wantRead() && !getSocket()->wantWrite()) {
+  if(!segmentPartComplete && eof) {
     throw DL_RETRY_EX(EX_GOT_EOF);
   }
 
@@ -198,10 +214,8 @@ bool DownloadCommand::executeInternal() {
       // If segment->getLength() == 0, the server doesn't provide
       // content length, but the client detected that download
       // completed.
-      if(getLogger()->info()) {
-        getLogger()->info(MSG_SEGMENT_DOWNLOAD_COMPLETED,
-                          util::itos(getCuid()).c_str());
-      }
+      A2_LOG_INFO(fmt(MSG_SEGMENT_DOWNLOAD_COMPLETED,
+                      getCuid()));
 #ifdef ENABLE_MESSAGE_DIGEST
 
       {
@@ -214,19 +228,16 @@ bool DownloadCommand::executeInternal() {
               !getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) &&
 #endif // ENABLE_BITTORRENT
              segment->isHashCalculated()) {
-            if(getLogger()->debug()) {
-              getLogger()->debug
-                ("Hash is available! index=%lu",
-                 static_cast<unsigned long>(segment->getIndex()));
-            }
+            A2_LOG_DEBUG(fmt("Hash is available! index=%lu",
+                             static_cast<unsigned long>(segment->getIndex())));
             validatePieceHash
               (segment, expectedPieceHash, segment->getHashString());
           } else {
-            messageDigestContext_->digestReset();
+            messageDigest_->reset();
             validatePieceHash
               (segment, expectedPieceHash,
-               MessageDigestHelper::digest
-               (messageDigestContext_.get(),
+               message_digest::hexDigest
+               (messageDigest_,
                 getPieceStorage()->getDiskAdaptor(),
                 segment->getPosition(),
                 segment->getLength()));
@@ -251,6 +262,7 @@ bool DownloadCommand::executeInternal() {
   } else {
     checkLowestDownloadSpeed();
     setWriteCheckSocketIf(getSocket(), getSocket()->wantWrite());
+    checkSocketRecvBuffer();
     getDownloadEngine()->addCommand(this);
     return false;
   }
@@ -263,11 +275,11 @@ void DownloadCommand::checkLowestDownloadSpeed() const
      startupIdleTime_) {
     unsigned int nowSpeed = peerStat_->calculateDownloadSpeed();
     if(nowSpeed <= lowestDownloadSpeedLimit_) {
-      throw DL_ABORT_EX2(StringFormat(EX_TOO_SLOW_DOWNLOAD_SPEED,
-                                      nowSpeed,
-                                      lowestDownloadSpeedLimit_,
-                                      getRequest()->getHost().c_str()).str(),
-                         downloadresultcode::TOO_SLOW_DOWNLOAD_SPEED);
+      throw DL_ABORT_EX2(fmt(EX_TOO_SLOW_DOWNLOAD_SPEED,
+                             nowSpeed,
+                             lowestDownloadSpeedLimit_,
+                             getRequest()->getHost().c_str()),
+                         error_code::TOO_SLOW_DOWNLOAD_SPEED);
     }
   }
 }
@@ -294,11 +306,11 @@ bool DownloadCommand::prepareForNextSegment() {
         getDownloadEngine()->getCheckIntegrityMan()->pushEntry(entry);
       }
     }
+#endif // ENABLE_MESSAGE_DIGEST
     // Following 2lines are needed for DownloadEngine to detect
     // completed RequestGroups without 1sec delay.
     getDownloadEngine()->setNoWait(true);
     getDownloadEngine()->setRefreshInterval(0);
-#endif // ENABLE_MESSAGE_DIGEST
     return true;
   } else {
     // The number of segments should be 1 in order to pass through the next
@@ -315,16 +327,17 @@ bool DownloadCommand::prepareForNextSegment() {
       }
       SharedHandle<Segment> nextSegment = getSegmentMan()->getSegmentWithIndex
         (getCuid(), tempSegment->getIndex()+1);
-      if(nextSegment.isNull()) {
+      if(!nextSegment) {
         nextSegment = getSegmentMan()->getCleanSegmentIfOwnerIsIdle
         (getCuid(), tempSegment->getIndex()+1);
       }
-      if(nextSegment.isNull() || nextSegment->getWrittenLength() > 0) {
+      if(!nextSegment || nextSegment->getWrittenLength() > 0) {
         // If nextSegment->getWrittenLength() > 0, current socket must
         // be closed because writing incoming data at
         // nextSegment->getWrittenLength() corrupts file.
         return prepareForRetry(0);
       } else {
+        checkSocketRecvBuffer();
         getDownloadEngine()->addCommand(this);
         return false;
       }
@@ -341,18 +354,19 @@ void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
                                         const std::string& actualPieceHash)
 {
   if(actualPieceHash == expectedPieceHash) {
-    getLogger()->info(MSG_GOOD_CHUNK_CHECKSUM, actualPieceHash.c_str());
+    A2_LOG_INFO(fmt(MSG_GOOD_CHUNK_CHECKSUM, actualPieceHash.c_str()));
     getSegmentMan()->completeSegment(getCuid(), segment);
   } else {
-    getLogger()->info(EX_INVALID_CHUNK_CHECKSUM,
-                      segment->getIndex(),
-                      util::itos(segment->getPosition(), true).c_str(),
-                      expectedPieceHash.c_str(),
-                      actualPieceHash.c_str());
+    A2_LOG_INFO(fmt(EX_INVALID_CHUNK_CHECKSUM,
+                    static_cast<unsigned long>(segment->getIndex()),
+                    util::itos(segment->getPosition(), true).c_str(),
+                    expectedPieceHash.c_str(),
+                    actualPieceHash.c_str()));
     segment->clear();
     getSegmentMan()->cancelSegment(getCuid());
     throw DL_RETRY_EX
-      (StringFormat("Invalid checksum index=%d", segment->getIndex()).str());
+      (fmt("Invalid checksum index=%lu",
+           static_cast<unsigned long>(segment->getIndex())));
   }
 }
 
@@ -361,7 +375,7 @@ void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
 void DownloadCommand::installStreamFilter
 (const SharedHandle<StreamFilter>& streamFilter)
 {
-  if(streamFilter.isNull()) {
+  if(!streamFilter) {
     return;
   }
   streamFilter->installDelegate(streamFilter_);
