@@ -43,16 +43,15 @@
 #include "DlAbortEx.h"
 #include "Option.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "Socket.h"
 #include "message.h"
 #include "prefs.h"
 #include "A2STR.h"
-#include "DownloadContext.h"
 #include "util.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
+#include "fmt.h"
+#include "SocketRecvBuffer.h"
+
 namespace aria2 {
 
 HttpInitiateConnectionCommand::HttpInitiateConnectionCommand
@@ -60,8 +59,9 @@ HttpInitiateConnectionCommand::HttpInitiateConnectionCommand
  const SharedHandle<Request>& req,
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
- DownloadEngine* e):
-  InitiateConnectionCommand(cuid, req, fileEntry, requestGroup, e) {}
+ DownloadEngine* e)
+  : InitiateConnectionCommand(cuid, req, fileEntry, requestGroup, e)
+{}
 
 HttpInitiateConnectionCommand::~HttpInitiateConnectionCommand() {}
 
@@ -71,20 +71,19 @@ Command* HttpInitiateConnectionCommand::createNextCommand
  const SharedHandle<Request>& proxyRequest)
 {
   Command* command;
-  if(!proxyRequest.isNull()) {
+  if(proxyRequest) {
     SharedHandle<SocketCore> pooledSocket =
       getDownloadEngine()->popPooledSocket
       (getRequest()->getHost(), getRequest()->getPort(),
        proxyRequest->getHost(), proxyRequest->getPort());
     std::string proxyMethod = resolveProxyMethod(getRequest()->getProtocol());
-    if(pooledSocket.isNull()) {
-      if(getLogger()->info()) {
-        getLogger()->info(MSG_CONNECTING_TO_SERVER,
-                          util::itos(getCuid()).c_str(), addr.c_str(), port);
-      }
+    if(!pooledSocket) {
+      A2_LOG_INFO(fmt(MSG_CONNECTING_TO_SERVER,
+                      getCuid(), addr.c_str(), port));
       createSocket();
       getSocket()->establishConnection(addr, port);
 
+      getRequest()->setConnectedAddrInfo(hostname, addr, port);
       if(proxyMethod == V_TUNNEL) {
         HttpProxyRequestCommand* c =
           new HttpProxyRequestCommand(getCuid(),
@@ -94,11 +93,12 @@ Command* HttpInitiateConnectionCommand::createNextCommand
                                       getDownloadEngine(),
                                       proxyRequest,
                                       getSocket());
-        c->setConnectedAddr(hostname, addr, port);
         command = c;
       } else if(proxyMethod == V_GET) {
+        SharedHandle<SocketRecvBuffer> socketRecvBuffer
+          (new SocketRecvBuffer(getSocket()));
         SharedHandle<HttpConnection> httpConnection
-          (new HttpConnection(getCuid(), getSocket()));
+          (new HttpConnection(getCuid(), getSocket(), socketRecvBuffer));
         HttpRequestCommand* c = new HttpRequestCommand(getCuid(),
                                                        getRequest(),
                                                        getFileEntry(),
@@ -106,7 +106,6 @@ Command* HttpInitiateConnectionCommand::createNextCommand
                                                        httpConnection,
                                                        getDownloadEngine(),
                                                        getSocket());
-        c->setConnectedAddr(hostname, addr, port);
         c->setProxyRequest(proxyRequest);
         command = c;
       } else {
@@ -114,8 +113,11 @@ Command* HttpInitiateConnectionCommand::createNextCommand
         throw DL_ABORT_EX("ERROR");
       }
     } else {
+      setConnectedAddrInfo(getRequest(), hostname, pooledSocket);
+      SharedHandle<SocketRecvBuffer> socketRecvBuffer
+        (new SocketRecvBuffer(pooledSocket));
       SharedHandle<HttpConnection> httpConnection
-        (new HttpConnection(getCuid(), pooledSocket));
+        (new HttpConnection(getCuid(), pooledSocket, socketRecvBuffer));
       HttpRequestCommand* c = new HttpRequestCommand(getCuid(),
                                                      getRequest(),
                                                      getFileEntry(),
@@ -132,27 +134,26 @@ Command* HttpInitiateConnectionCommand::createNextCommand
     SharedHandle<SocketCore> pooledSocket =
       getDownloadEngine()->popPooledSocket
       (resolvedAddresses, getRequest()->getPort());
-    if(pooledSocket.isNull()) {
-      if(getLogger()->info()) {
-        getLogger()->info(MSG_CONNECTING_TO_SERVER,
-                          util::itos(getCuid()).c_str(), addr.c_str(), port);
-      }
+    if(!pooledSocket) {
+      A2_LOG_INFO(fmt(MSG_CONNECTING_TO_SERVER,
+                      getCuid(), addr.c_str(), port));
       createSocket();
       getSocket()->establishConnection(addr, port);
+      getRequest()->setConnectedAddrInfo(hostname, addr, port);
     } else {
       setSocket(pooledSocket);
+      setConnectedAddrInfo(getRequest(), hostname, pooledSocket);
     }
+    SharedHandle<SocketRecvBuffer> socketRecvBuffer
+      (new SocketRecvBuffer(getSocket()));
     SharedHandle<HttpConnection> httpConnection
-      (new HttpConnection(getCuid(), getSocket()));
+      (new HttpConnection(getCuid(), getSocket(), socketRecvBuffer));
     HttpRequestCommand* c =
       new HttpRequestCommand(getCuid(), getRequest(), getFileEntry(),
                              getRequestGroup(),
                              httpConnection,
                              getDownloadEngine(),
                              getSocket());
-    if(pooledSocket.isNull()) {
-      c->setConnectedAddr(hostname, addr, port);
-    }
     command = c;
   }
   return command;
