@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_HTTP_SERVER_H_
-#define _D_HTTP_SERVER_H_
+#ifndef D_HTTP_SERVER_H
+#define D_HTTP_SERVER_H
 
 #include "common.h"
 
@@ -50,15 +50,15 @@ class SocketCore;
 class HttpHeader;
 class HttpHeaderProcessor;
 class DownloadEngine;
-class Logger;
+class SocketRecvBuffer;
 
 class HttpServer {
 private:
   SharedHandle<SocketCore> socket_;
+  SharedHandle<SocketRecvBuffer> socketRecvBuffer_;
   SocketBuffer socketBuffer_;
   DownloadEngine* e_;
   SharedHandle<HttpHeaderProcessor> headerProcessor_;
-  Logger* logger_;
   SharedHandle<HttpHeader> lastRequestHeader_;
   uint64_t lastContentLength_;
   std::stringstream lastBody_;
@@ -79,6 +79,8 @@ public:
 
   std::string getBody() const;
 
+  const std::string& getMethod() const;
+
   const std::string& getRequestPath() const;
 
   void feedResponse(const std::string& text, const std::string& contentType);
@@ -91,11 +93,7 @@ public:
   bool authenticate();
 
   void setUsernamePassword
-  (const std::string& username, const std::string& password)
-  {
-    username_ = username;
-    password_ = password;
-  }
+  (const std::string& username, const std::string& password);
 
   ssize_t sendResponse();
 
@@ -120,8 +118,13 @@ public:
   void disableGZip() { gzip_ = false; }
 
   uint64_t getContentLength() const { return lastContentLength_; }
+
+  const SharedHandle<SocketRecvBuffer>& getSocketRecvBuffer() const
+  {
+    return socketRecvBuffer_;
+  }
 };
 
 } // namespace aria2
 
-#endif // _D_HTTP_SERVER_H_
+#endif // D_HTTP_SERVER_H
