@@ -38,6 +38,7 @@
 #include "HttpServer.h"
 #include "HttpHeader.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "RequestGroup.h"
 #include "RequestGroupMan.h"
 #include "HttpServerBodyCommand.h"
@@ -46,42 +47,45 @@
 #include "prefs.h"
 #include "Option.h"
 #include "util.h"
-#include "DownloadContext.h"
 #include "wallclock.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
+#include "fmt.h"
+#include "SocketRecvBuffer.h"
 
 namespace aria2 {
 
-HttpServerCommand::HttpServerCommand(cuid_t cuid, DownloadEngine* e,
-                                     const SharedHandle<SocketCore>& socket):
-  Command(cuid),
-  e_(e),
-  socket_(socket),
-  httpServer_(new HttpServer(socket, e))
+HttpServerCommand::HttpServerCommand
+(cuid_t cuid,
+ DownloadEngine* e,
+ const SharedHandle<SocketCore>& socket)
+  : Command(cuid),
+    e_(e),
+    socket_(socket),
+    httpServer_(new HttpServer(socket, e))
 {
   setStatus(Command::STATUS_ONESHOT_REALTIME);
   e_->addSocketForReadCheck(socket_, this);
-  httpServer_->setUsernamePassword(e_->getOption()->get(PREF_XML_RPC_USER),
-                                   e_->getOption()->get(PREF_XML_RPC_PASSWD));
-#ifdef HAVE_LIBZ
+  httpServer_->setUsernamePassword(e_->getOption()->get(PREF_RPC_USER),
+                                   e_->getOption()->get(PREF_RPC_PASSWD));
+#ifdef HAVE_ZLIB
   httpServer_->enableGZip();
-#else // !HAVE_LIBZ
+#else // !HAVE_ZLIB
   httpServer_->disableGZip();
-#endif // !HAVE_LIBZ
+#endif // !HAVE_ZLIB
+  checkSocketRecvBuffer();
 }
 
-HttpServerCommand::HttpServerCommand(cuid_t cuid,
-                                     const SharedHandle<HttpServer>& httpServer,
-                                     DownloadEngine* e,
-                                     const SharedHandle<SocketCore>& socket):
-  Command(cuid),
-  e_(e),
-  socket_(socket),
-  httpServer_(httpServer)
+HttpServerCommand::HttpServerCommand
+(cuid_t cuid,
+ const SharedHandle<HttpServer>& httpServer,
+ DownloadEngine* e,
+ const SharedHandle<SocketCore>& socket)
+  : Command(cuid),
+    e_(e),
+    socket_(socket),
+    httpServer_(httpServer)
 {
   e_->addSocketForReadCheck(socket_, this);
+  checkSocketRecvBuffer();
 }
 
 HttpServerCommand::~HttpServerCommand()
@@ -89,19 +93,28 @@ HttpServerCommand::~HttpServerCommand()
   e_->deleteSocketForReadCheck(socket_, this);
 }
 
+void HttpServerCommand::checkSocketRecvBuffer()
+{
+  if(!httpServer_->getSocketRecvBuffer()->bufferEmpty()) {
+    setStatus(Command::STATUS_ONESHOT_REALTIME);
+    e_->setNoWait(true);
+  }
+}
+
 bool HttpServerCommand::execute()
 {
   if(e_->getRequestGroupMan()->downloadFinished() || e_->isHaltRequested()) {
     return true;
   }
   try {
-    if(socket_->isReadable(0)) {
+    if(socket_->isReadable(0) ||
+       !httpServer_->getSocketRecvBuffer()->bufferEmpty()) {
       timeoutTimer_ = global::wallclock;
       SharedHandle<HttpHeader> header;
 
       header = httpServer_->receiveRequest();
 
-      if(header.isNull()) {
+      if(!header) {
         e_->addCommand(this);
         return false;
       }
@@ -117,12 +130,12 @@ bool HttpServerCommand::execute()
         return true;
       }
       if(static_cast<uint64_t>
-         (e_->getOption()->getAsInt(PREF_XML_RPC_MAX_REQUEST_SIZE)) <
+         (e_->getOption()->getAsInt(PREF_RPC_MAX_REQUEST_SIZE)) <
          httpServer_->getContentLength()) {
-        getLogger()->info("Request too long. ContentLength=%s."
-                          " See --xml-rpc-max-request-size option to loose"
-                          " this limitation.",
-                          util::uitos(httpServer_->getContentLength()).c_str());
+        A2_LOG_INFO(fmt("Request too long. ContentLength=%s."
+                        " See --rpc-max-request-size option to loose"
+                        " this limitation.",
+                        util::uitos(httpServer_->getContentLength()).c_str()));
         return true;
       }
       Command* command = new HttpServerBodyCommand(getCuid(), httpServer_, e_,
@@ -132,7 +145,7 @@ bool HttpServerCommand::execute()
       return true;
     } else {
       if(timeoutTimer_.difference(global::wallclock) >= 30) {
-        getLogger()->info("HTTP request timeout.");
+        A2_LOG_INFO("HTTP request timeout.");
         return true;
       } else {
         e_->addCommand(this);
@@ -140,10 +153,9 @@ bool HttpServerCommand::execute()
       }
     }
   } catch(RecoverableException& e) {
-    if(getLogger()->info()) {
-      getLogger()->info("CUID#%s - Error occurred while reading HTTP request",
-                        e, util::itos(getCuid()).c_str());
-    }
+    A2_LOG_INFO_EX(fmt("CUID#%lld - Error occurred while reading HTTP request",
+                       getCuid()),
+                   e);
     return true;
   }
 
