@@ -40,7 +40,7 @@
 #include "A2STR.h"
 #include "util.h"
 #include "help_tags.h"
-#include "StringFormat.h"
+#include "a2functional.h"
 #include "File.h"
 
 namespace aria2 {
@@ -53,7 +53,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_ALLOW_OVERWRITE,
                                     TEXT_ALLOW_OVERWRITE,
-                                    A2_V_FALSE));
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_FILE);
     handlers.push_back(op);
@@ -62,7 +63,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_ALLOW_PIECE_LENGTH_CHANGE,
                                     TEXT_ALLOW_PIECE_LENGTH_CHANGE,
-                                    A2_V_FALSE));
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -87,6 +89,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+#if defined HAVE_ARES_SET_SERVERS && HAVE_ARES_ADDR_NODE
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+                                   (PREF_ASYNC_DNS_SERVER,
+                                    TEXT_ASYNC_DNS_SERVER,
+                                    NO_DEFAULT_VALUE));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
+#endif // HAVE_ARES_SET_SERVERS && HAVE_ARES_ADDR_NODE
 #endif // ENABLE_ASYNC_DNS
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
@@ -136,7 +148,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_CONF_PATH,
                                     TEXT_CONF_PATH,
-                                    util::getHomeDir()+"/.aria2/aria2.conf"));
+                                    util::getHomeDir()+"/.aria2/aria2.conf",
+                                    PATH_TO_FILE));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -144,8 +157,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_CONTINUE,
                                     TEXT_CONTINUE,
-                                    A2_V_FALSE, // TODO ommit?
-                                    OptionHandler::NO_ARG,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG,
                                     'c'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
@@ -157,8 +170,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_DAEMON,
                                     TEXT_DAEMON,
                                     A2_V_FALSE,
-                                    OptionHandler::NO_ARG,
-                                    'D')); // TODO ommit?
+                                    OptionHandler::OPT_ARG,
+                                    'D'));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -167,7 +180,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_DIR,
                                     TEXT_DIR,
                                     File::getCurrentDir(),
-                                    A2STR::NIL,
+                                    PATH_TO_DIR,
                                     OptionHandler::REQ_ARG,
                                     'd'));
     op->addTag(TAG_BASIC);
@@ -215,17 +228,28 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
 #endif // ENABLE_DIRECT_IO
-#ifdef ENABLE_XML_RPC
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
-                                   (PREF_ENABLE_XML_RPC,
-                                    TEXT_ENABLE_XML_RPC,
+                                   (PREF_ENABLE_RPC,
+                                    TEXT_ENABLE_RPC,
                                     A2_V_FALSE,
                                     OptionHandler::OPT_ARG));
-    op->addTag(TAG_XML_RPC);
+    op->addTag(TAG_RPC);
+    handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new BooleanOptionHandler
+                                    (PREF_ENABLE_XML_RPC,
+                                     TEXT_ENABLE_XML_RPC,
+                                     NO_DEFAULT_VALUE,
+                                     OptionHandler::OPT_ARG)),
+        PREF_ENABLE_RPC));
+    op->addTag(TAG_RPC);
     handlers.push_back(op);
   }
-#endif // ENABLE_XML_RPC
   {
     std::string params[] = {
 #ifdef HAVE_EPOLL
@@ -297,7 +321,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_INPUT_FILE,
                                     TEXT_INPUT_FILE,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME,-",
+                                    PATH_TO_FILE_STDIN,
                                     OptionHandler::REQ_ARG,
                                     'i'));
     op->addTag(TAG_BASIC);
@@ -318,7 +342,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_LOG,
                                     TEXT_LOG,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME,-",
+                                    PATH_TO_FILE_STDOUT,
                                     OptionHandler::REQ_ARG,
                                     'l'));
     op->addTag(TAG_BASIC);
@@ -416,7 +440,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_NO_CONF,
                                     TEXT_NO_CONF,
                                     A2_V_FALSE,
-                                    OptionHandler::NO_ARG));
+                                    OptionHandler::OPT_ARG));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -435,7 +459,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_DOWNLOAD_COMPLETE,
                                     TEXT_ON_DOWNLOAD_COMPLETE,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -445,7 +469,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_DOWNLOAD_ERROR,
                                     TEXT_ON_DOWNLOAD_ERROR,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -455,7 +479,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_DOWNLOAD_PAUSE,
                                     TEXT_ON_DOWNLOAD_PAUSE,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -465,7 +489,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_DOWNLOAD_START,
                                     TEXT_ON_DOWNLOAD_START,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -475,7 +499,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_DOWNLOAD_STOP,
                                     TEXT_ON_DOWNLOAD_STOP,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -505,7 +529,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_REALTIME_CHUNK_CHECKSUM,
                                     TEXT_REALTIME_CHUNK_CHECKSUM,
-                                    A2_V_TRUE));
+                                    A2_V_TRUE,
+                                    OptionHandler::OPT_ARG));
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
@@ -524,18 +549,26 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_SAVE_SESSION,
                                     TEXT_SAVE_SESSION,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_SELECT_LEAST_USED_HOST,
-                                    NO_DEFAULT_VALUE,
+                                    NO_DESCRIPTION,
                                     A2_V_TRUE));
     op->hide();
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+                                   (PREF_SHOW_CONSOLE_READOUT,
+                                    TEXT_SHOW_CONSOLE_READOUT,
+                                    A2_V_TRUE));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
                                    (PREF_STOP,
@@ -554,49 +587,112 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
-#ifdef ENABLE_XML_RPC
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
-                                   (PREF_XML_RPC_LISTEN_ALL,
-                                    TEXT_XML_RPC_LISTEN_ALL,
+                                   (PREF_RPC_LISTEN_ALL,
+                                    TEXT_RPC_LISTEN_ALL,
                                     A2_V_FALSE,
                                     OptionHandler::OPT_ARG));
-    op->addTag(TAG_XML_RPC);
+    op->addTag(TAG_RPC);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
-                                   (PREF_XML_RPC_LISTEN_PORT,
-                                    TEXT_XML_RPC_LISTEN_PORT,
+                                   (PREF_RPC_LISTEN_PORT,
+                                    TEXT_RPC_LISTEN_PORT,
                                     "6800",
                                     1024, UINT16_MAX));
-    op->addTag(TAG_XML_RPC);
+    op->addTag(TAG_RPC);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
-                                   (PREF_XML_RPC_MAX_REQUEST_SIZE,
-                                    TEXT_XML_RPC_MAX_REQUEST_SIZE,
+                                   (PREF_RPC_MAX_REQUEST_SIZE,
+                                    TEXT_RPC_MAX_REQUEST_SIZE,
                                     "2M",
                                     0));
-    op->addTag(TAG_XML_RPC);
+    op->addTag(TAG_RPC);
     handlers.push_back(op);
   }                         
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
-                                   (PREF_XML_RPC_USER,
-                                    TEXT_XML_RPC_USER));
-    op->addTag(TAG_XML_RPC);
+                                   (PREF_RPC_USER,
+                                    TEXT_RPC_USER));
+    op->addTag(TAG_RPC);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
-                                   (PREF_XML_RPC_PASSWD,
-                                    TEXT_XML_RPC_PASSWD));
-    op->addTag(TAG_XML_RPC);
+                                   (PREF_RPC_PASSWD,
+                                    TEXT_RPC_PASSWD));
+    op->addTag(TAG_RPC);
+    op->setEraseAfterParse(true);
+    handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new BooleanOptionHandler
+                                    (PREF_XML_RPC_LISTEN_ALL,
+                                     TEXT_XML_RPC_LISTEN_ALL,
+                                     NO_DEFAULT_VALUE,
+                                     OptionHandler::OPT_ARG)),
+        PREF_RPC_LISTEN_ALL));
+    op->addTag(TAG_RPC);
+    handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new NumberOptionHandler
+                                    (PREF_XML_RPC_LISTEN_PORT,
+                                     TEXT_XML_RPC_LISTEN_PORT,
+                                     NO_DEFAULT_VALUE,
+                                     1024, UINT16_MAX)),
+        PREF_RPC_LISTEN_PORT));
+    op->addTag(TAG_RPC);
+    handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+     SharedHandle<OptionHandler> op
+       (new DeprecatedOptionHandler
+        (SharedHandle<OptionHandler>(new UnitNumberOptionHandler
+                                     (PREF_XML_RPC_MAX_REQUEST_SIZE,
+                                      TEXT_XML_RPC_MAX_REQUEST_SIZE,
+                                      NO_DEFAULT_VALUE,
+                                      0)),
+         PREF_RPC_MAX_REQUEST_SIZE));
+     op->addTag(TAG_RPC);
+     handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new DefaultOptionHandler
+                                    (PREF_XML_RPC_USER,
+                                     TEXT_XML_RPC_USER)),
+        PREF_RPC_USER));
+    op->addTag(TAG_RPC);
+    op->setEraseAfterParse(true);
+    handlers.push_back(op);
+  }
+  {
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new DefaultOptionHandler
+                                    (PREF_XML_RPC_PASSWD,
+                                     TEXT_XML_RPC_PASSWD)),
+        PREF_RPC_PASSWD));
+    op->addTag(TAG_RPC);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }                         
-#endif // ENABLE_XML_RPC
   // HTTP/FTP options
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
@@ -653,8 +749,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_NO_NETRC,
                                     TEXT_NO_NETRC,
-                                    A2_V_FALSE, // TODO ommit?
-                                    OptionHandler::NO_ARG,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG,
                                     'n'));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
@@ -665,7 +761,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_OUT,
                                     TEXT_OUT,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME",
+                                    PATH_TO_FILE,
                                     OptionHandler::REQ_ARG,
                                     'o'));
     op->addTag(TAG_BASIC);
@@ -685,6 +781,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new NumberOptionHandler
+                                   (PREF_RETRY_WAIT,
+                                    TEXT_RETRY_WAIT,
+                                    "0",
+                                    0, 600));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_REUSE_URI,
@@ -709,7 +815,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_SERVER_STAT_IF,
                                     TEXT_SERVER_STAT_IF,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
@@ -719,7 +825,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_SERVER_STAT_OF,
                                     TEXT_SERVER_STAT_OF,
                                     NO_DEFAULT_VALUE,
-                                    "FILENAME"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
@@ -783,7 +889,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_CA_CERTIFICATE,
                                     TEXT_CA_CERTIFICATE,
-                                    CA_BUNDLE));
+                                    CA_BUNDLE,
+                                    PATH_TO_FILE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
     handlers.push_back(op);
@@ -791,7 +898,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_CERTIFICATE,
-                                    TEXT_CERTIFICATE));
+                                    TEXT_CERTIFICATE,
+                                    NO_DEFAULT_VALUE,
+                                    PATH_TO_FILE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
     handlers.push_back(op);
@@ -866,6 +975,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_HTTP_PASSWD));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_HTTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -874,6 +984,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_HTTP_USER));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_HTTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -881,7 +992,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_LOAD_COOKIES,
                                     TEXT_LOAD_COOKIES,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/file"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_HTTP);
     op->addTag(TAG_COOKIE);
@@ -899,7 +1010,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_PRIVATE_KEY,
-                                    TEXT_PRIVATE_KEY));
+                                    TEXT_PRIVATE_KEY,
+                                    NO_DEFAULT_VALUE,
+                                    PATH_TO_FILE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
     handlers.push_back(op);
@@ -916,7 +1029,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_SAVE_COOKIES,
                                     TEXT_SAVE_COOKIES,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/file"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_COOKIE);
     handlers.push_back(op);
@@ -948,6 +1061,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_FTP_PASSWD));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }    
   {
@@ -984,6 +1098,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_FTP_USER));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -991,7 +1106,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                           (PREF_NETRC_PATH,
                                            NO_DESCRIPTION,
                                            util::getHomeDir()+"/.netrc",
-                                           "/PATH/TO/NETRC"));
+                                           PATH_TO_FILE));
     op->hide();
     handlers.push_back(op);
   }
@@ -1010,6 +1125,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_HTTP_PROXY_PASSWD,
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_HTTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1018,6 +1134,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_HTTP_PROXY_USER,
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_HTTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1036,6 +1153,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1045,6 +1163,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1061,6 +1180,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_FTP_PROXY_PASSWD,
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_FTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1069,6 +1189,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_FTP_PROXY_USER,
                                     NO_DEFAULT_VALUE));
     op->addTag(TAG_FTP);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1089,6 +1210,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1099,6 +1221,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
+    op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
   {
@@ -1138,8 +1261,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_SHOW_FILES,
                                     TEXT_SHOW_FILES,
-                                    A2_V_FALSE, // TODO ommit?
-                                    OptionHandler::NO_ARG,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG,
                                     'S'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
@@ -1262,7 +1385,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_BT_REQUIRE_CRYPTO,
                                     TEXT_BT_REQUIRE_CRYPTO,
-                                    A2_V_FALSE));
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -1372,7 +1496,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_DHT_FILE_PATH,
                                     TEXT_DHT_FILE_PATH,
                                     util::getHomeDir()+"/.aria2/dht.dat",
-                                    "/PATH/TO/DHT_DAT"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -1381,7 +1505,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_DHT_FILE_PATH6,
                                     TEXT_DHT_FILE_PATH6,
                                     util::getHomeDir()+"/.aria2/dht6.dat",
-                                    "/PATH/TO/DHT_DAT"));
+                                    PATH_TO_FILE));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -1505,7 +1629,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                    (PREF_ON_BT_DOWNLOAD_COMPLETE,
                                     TEXT_ON_BT_DOWNLOAD_COMPLETE,
                                     NO_DEFAULT_VALUE,
-                                    "/path/to/command"));
+                                    PATH_TO_COMMAND));
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_HOOK);
     handlers.push_back(op);
@@ -1568,6 +1692,14 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+                                   (PREF_METALINK_BASE_URI,
+                                    TEXT_METALINK_BASE_URI,
+                                    NO_DEFAULT_VALUE));
+    op->addTag(TAG_METALINK);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_METALINK_ENABLE_UNIQUE_PROTOCOL,
@@ -1652,27 +1784,28 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
   }
   // Help Option
   {
+    static std::string tags[] = {
+      TAG_BASIC,
+      TAG_ADVANCED,
+      TAG_HTTP,
+      TAG_HTTPS,
+      TAG_FTP,
+      TAG_METALINK,
+      TAG_BITTORRENT,
+      TAG_COOKIE,
+      TAG_HOOK,
+      TAG_FILE,
+      TAG_RPC,
+      TAG_EXPERIMENTAL,
+      TAG_HELP,
+      TAG_ALL
+    };
+    static std::string tagsStr = strjoin(vbegin(tags), vend(tags), ", ");
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    ("help",
                                     TEXT_HELP,
                                     TAG_BASIC,
-                                    StringFormat
-                                    ("%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, "
-                                     "%s, %s, %s, %s",
-                                     TAG_BASIC,
-                                     TAG_ADVANCED,
-                                     TAG_HTTP,
-                                     TAG_HTTPS,
-                                     TAG_FTP,
-                                     TAG_METALINK,
-                                     TAG_BITTORRENT,
-                                     TAG_COOKIE,
-                                     TAG_HOOK,
-                                     TAG_FILE,
-                                     TAG_XML_RPC,
-                                     TAG_EXPERIMENTAL,
-                                     TAG_HELP,
-                                     TAG_ALL).str(),
+                                    tagsStr,
                                     OptionHandler::OPT_ARG,
                                     'h'));
     op->addTag(TAG_BASIC);
