@@ -42,6 +42,7 @@
 #include <fstream>
 #include <numeric>
 #include <algorithm>
+#include <utility>
 
 #include "BtProgressInfoFile.h"
 #include "RecoverableException.h"
@@ -66,41 +67,43 @@
 #include "util.h"
 #include "Command.h"
 #include "FileEntry.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "FileAllocationEntry.h"
 #include "CheckIntegrityEntry.h"
 #include "Segment.h"
 #include "DlAbortEx.h"
 #include "uri.h"
+#include "Triplet.h"
+#include "Signature.h"
 
 namespace aria2 {
 
 RequestGroupMan::RequestGroupMan
 (const std::vector<SharedHandle<RequestGroup> >& requestGroups,
  unsigned int maxSimultaneousDownloads,
- const Option* option):
-  reservedGroups_(requestGroups.begin(), requestGroups.end()),
-  logger_(LogFactory::getInstance()),
-  maxSimultaneousDownloads_(maxSimultaneousDownloads),
-  option_(option),
-  serverStatMan_(new ServerStatMan()),
-  maxOverallDownloadSpeedLimit_
-  (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
-  maxOverallUploadSpeedLimit_(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT)),
-  xmlRpc_(option->getAsBool(PREF_ENABLE_XML_RPC)),
-  queueCheck_(true),
-  removedErrorResult_(0),
-  removedLastErrorResult_(downloadresultcode::FINISHED),
-  maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT))
+ const Option* option)
+  : reservedGroups_(requestGroups.begin(), requestGroups.end()),
+    maxSimultaneousDownloads_(maxSimultaneousDownloads),
+    option_(option),
+    serverStatMan_(new ServerStatMan()),
+    maxOverallDownloadSpeedLimit_
+    (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
+    maxOverallUploadSpeedLimit_(option->getAsInt
+                                (PREF_MAX_OVERALL_UPLOAD_LIMIT)),
+    rpc_(option->getAsBool(PREF_ENABLE_RPC)),
+    queueCheck_(true),
+    removedErrorResult_(0),
+    removedLastErrorResult_(error_code::FINISHED),
+    maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT))
 {}
 
+RequestGroupMan::~RequestGroupMan() {}
+
 bool RequestGroupMan::downloadFinished()
 {
-#ifdef ENABLE_XML_RPC
-  if(xmlRpc_) {
+  if(rpc_) {
     return false;
   }
-#endif // ENABLE_XML_RPC
   return requestGroups_.empty() && reservedGroups_.empty();
 }
 
@@ -163,8 +166,9 @@ SharedHandle<RequestGroup> RequestGroupMan::getRequestGroup(size_t index) const
   }
 }
 
+namespace {
 template<typename Iterator>
-static Iterator findByGID(Iterator first, Iterator last, gid_t gid)
+Iterator findByGID(Iterator first, Iterator last, a2_gid_t gid)
 {
   for(; first != last; ++first) {
     if((*first)->getGID() == gid) {
@@ -173,9 +177,10 @@ static Iterator findByGID(Iterator first, Iterator last, gid_t gid)
   }
   return first;
 }
+} // namespace
 
 SharedHandle<RequestGroup>
-RequestGroupMan::findRequestGroup(gid_t gid) const
+RequestGroupMan::findRequestGroup(a2_gid_t gid) const
 {
   std::deque<SharedHandle<RequestGroup> >::const_iterator i =
     findByGID(requestGroups_.begin(), requestGroups_.end(), gid);
@@ -187,7 +192,7 @@ RequestGroupMan::findRequestGroup(gid_t gid) const
 }
 
 SharedHandle<RequestGroup>
-RequestGroupMan::findReservedGroup(gid_t gid) const
+RequestGroupMan::findReservedGroup(a2_gid_t gid) const
 {
   std::deque<SharedHandle<RequestGroup> >::const_iterator i =
     findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
@@ -199,14 +204,14 @@ RequestGroupMan::findReservedGroup(gid_t gid) const
 }
 
 size_t RequestGroupMan::changeReservedGroupPosition
-(gid_t gid, int pos, HOW how)
+(a2_gid_t gid, int pos, HOW how)
 {
   std::deque<SharedHandle<RequestGroup> >::iterator i =
     findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
   if(i == reservedGroups_.end()) {
     throw DL_ABORT_EX
-      (StringFormat("GID#%s not found in the waiting queue.",
-                    util::itos(gid).c_str()).str());
+      (fmt("GID#%s not found in the waiting queue.",
+           util::itos(gid).c_str()));
   }
   SharedHandle<RequestGroup> rg = *i;
   const size_t maxPos = reservedGroups_.size()-1;
@@ -242,7 +247,7 @@ size_t RequestGroupMan::changeReservedGroupPosition
   return pos;
 }
 
-bool RequestGroupMan::removeReservedGroup(gid_t gid)
+bool RequestGroupMan::removeReservedGroup(a2_gid_t gid)
 {
   std::deque<SharedHandle<RequestGroup> >::iterator i =
     findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
@@ -254,23 +259,26 @@ bool RequestGroupMan::removeReservedGroup(gid_t gid)
   }
 }
 
-static void executeStopHook
-(const SharedHandle<DownloadResult>& result, const Option* option)
+namespace {
+
+void executeStopHook
+(const SharedHandle<RequestGroup>& group,
+ const Option* option,
+ error_code::Value result)
 {
-  if(result->result == downloadresultcode::FINISHED &&
+  if(result == error_code::FINISHED &&
      !option->blank(PREF_ON_DOWNLOAD_COMPLETE)) {
-    util::executeHook(option->get(PREF_ON_DOWNLOAD_COMPLETE),
-                      util::itos(result->gid));
-  } else if(result->result != downloadresultcode::IN_PROGRESS &&
+    util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_COMPLETE);
+  } else if(result != error_code::IN_PROGRESS &&
             !option->blank(PREF_ON_DOWNLOAD_ERROR)) {
-    util::executeHook(option->get(PREF_ON_DOWNLOAD_ERROR),
-                      util::itos(result->gid));
+    util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_ERROR);
   } else if(!option->blank(PREF_ON_DOWNLOAD_STOP)) {
-    util::executeHook(option->get(PREF_ON_DOWNLOAD_STOP),
-                      util::itos(result->gid));
+    util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_STOP);
   }
 }
 
+} // namespace
+
 namespace {
 class ProcessStoppedRequestGroup {
 private:
@@ -283,14 +291,14 @@ private:
   {
     SharedHandle<Signature> sig =
       group->getDownloadContext()->getSignature();
-    if(!sig.isNull() && !sig->getBody().empty()) {
+    if(sig && !sig->getBody().empty()) {
       // filename of signature file is the path to download file followed by
       // ".sig".
       std::string signatureFile = group->getFirstFilePath()+".sig";
       if(sig->save(signatureFile)) {
-        logger_->notice(MSG_SIGNATURE_SAVED, signatureFile.c_str());
+        A2_LOG_NOTICE(fmt(MSG_SIGNATURE_SAVED, signatureFile.c_str()));
       } else {
-        logger_->notice(MSG_SIGNATURE_NOT_SAVED, signatureFile.c_str());
+        A2_LOG_NOTICE(fmt(MSG_SIGNATURE_NOT_SAVED, signatureFile.c_str()));
       }
     }
   }
@@ -298,11 +306,11 @@ public:
   ProcessStoppedRequestGroup
   (DownloadEngine* e,
    std::deque<SharedHandle<DownloadResult> >& downloadResults,
-   std::deque<SharedHandle<RequestGroup> >& reservedGroups):
-    e_(e),
-    downloadResults_(downloadResults),
-    reservedGroups_(reservedGroups),
-    logger_(LogFactory::getInstance()) {}
+   std::deque<SharedHandle<RequestGroup> >& reservedGroups)
+    : e_(e),
+      downloadResults_(downloadResults),
+      reservedGroups_(reservedGroups)
+  {}
 
   void operator()(const SharedHandle<RequestGroup>& group)
   {
@@ -333,21 +341,21 @@ public:
           std::vector<SharedHandle<RequestGroup> > nextGroups;
           group->postDownloadProcessing(nextGroups);
           if(!nextGroups.empty()) {
-            if(logger_->debug()) {
-              logger_->debug
-                ("Adding %lu RequestGroups as a result of PostDownloadHandler.",
-                 static_cast<unsigned long>(nextGroups.size()));
-            }
+            A2_LOG_DEBUG
+              (fmt("Adding %lu RequestGroups as a result of"
+                   " PostDownloadHandler.",
+                   static_cast<unsigned long>(nextGroups.size())));
             e_->getRequestGroupMan()->insertReservedGroup(0, nextGroups);
           }
         } else {
-          logger_->notice("Download GID#%s not complete: %s",
-                          util::itos(group->getGID()).c_str(),
-                          group->getDownloadContext()->getBasePath().c_str());
+          A2_LOG_NOTICE
+            (fmt("Download GID#%s not complete: %s",
+                 util::itos(group->getGID()).c_str(),
+                 group->getDownloadContext()->getBasePath().c_str()));
           group->saveControlFile();
         }
       } catch(RecoverableException& ex) {
-        logger_->error(EX_EXCEPTION_CAUGHT, ex);
+        A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
       }
       if(group->isPauseRequested()) {
         reservedGroups_.push_front(group);
@@ -360,13 +368,13 @@ public:
       } else {
         SharedHandle<DownloadResult> dr = group->createDownloadResult();
         e_->getRequestGroupMan()->addDownloadResult(dr);
+        executeStopHook(group, e_->getOption(), dr->result);
         group->releaseRuntimeResource(e_);
-        executeStopHook(dr, e_->getOption());
       }
     }
   }
 };
-}
+} // namespace
 
 namespace {
 class CollectServerStat {
@@ -381,7 +389,7 @@ public:
     if(group->getNumCommand() == 0) {
       // Collect statistics during download in PeerStats and update/register
       // ServerStatMan
-      if(!group->getSegmentMan().isNull()) {
+      if(group->getSegmentMan()) {
         bool singleConnection =
           group->getSegmentMan()->getPeerStats().size() == 1;
         const std::vector<SharedHandle<PeerStat> >& peerStats =
@@ -410,7 +418,7 @@ public:
     }    
   }
 };
-}
+} // namespace
 
 namespace {
 class FindStoppedRequestGroup {
@@ -419,7 +427,7 @@ public:
     return group->getNumCommand() == 0;
   }
 };
-}
+} // namespace
 
 void RequestGroupMan::updateServerStat()
 {
@@ -446,10 +454,8 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 
   size_t numRemoved = numPrev-requestGroups_.size();
   if(numRemoved > 0) {
-    if(logger_->debug()) {
-      logger_->debug("%lu RequestGroup(s) deleted.",
-                     static_cast<unsigned long>(numRemoved));
-    }
+    A2_LOG_DEBUG(fmt("%lu RequestGroup(s) deleted.",
+                     static_cast<unsigned long>(numRemoved)));
   }
 }
 
@@ -458,24 +464,26 @@ void RequestGroupMan::configureRequestGroup
 {
   const std::string& uriSelectorValue = option_->get(PREF_URI_SELECTOR);
   if(uriSelectorValue == V_FEEDBACK) {
-    requestGroup->setURISelector
-      (SharedHandle<URISelector>(new FeedbackURISelector(serverStatMan_)));
+    SharedHandle<URISelector> sel(new FeedbackURISelector(serverStatMan_));
+    requestGroup->setURISelector(sel);
   } else if(uriSelectorValue == V_INORDER) {
-    requestGroup->setURISelector
-      (SharedHandle<URISelector>(new InOrderURISelector()));
+    SharedHandle<URISelector> sel(new InOrderURISelector());
+    requestGroup->setURISelector(sel);
   } else if(uriSelectorValue == V_ADAPTIVE) {
-    requestGroup->setURISelector
-      (SharedHandle<URISelector>(new AdaptiveURISelector(serverStatMan_,
-                                                         requestGroup.get())));
+    SharedHandle<URISelector> sel(new AdaptiveURISelector(serverStatMan_,
+                                                          requestGroup.get()));
+    requestGroup->setURISelector(sel);
   }
 }
 
-static void createInitialCommand(const SharedHandle<RequestGroup>& requestGroup,
-                                 std::vector<Command*>& commands,
-                                 DownloadEngine* e)
+namespace {
+void createInitialCommand(const SharedHandle<RequestGroup>& requestGroup,
+                          std::vector<Command*>& commands,
+                          DownloadEngine* e)
 {
   requestGroup->createInitialCommand(commands, e);
 }
+} // namespace
 
 void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
 {
@@ -508,30 +516,28 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       ++count;
       e->addCommand(commands);
       commands.clear();
-      util::executeHookByOptName
-        (groupToAdd, e->getOption(), PREF_ON_DOWNLOAD_START);
     } catch(RecoverableException& ex) {
-      logger_->error(EX_EXCEPTION_CAUGHT, ex);
-      if(logger_->debug()) {
-        logger_->debug("Deleting temporal commands.");
-      }
+      A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
+      A2_LOG_DEBUG("Deleting temporal commands.");
       std::for_each(commands.begin(), commands.end(), Deleter());
       commands.clear();
-      if(logger_->debug()) {
-        logger_->debug("Commands deleted");
-      }
-      groupToAdd->releaseRuntimeResource(e);
-      addDownloadResult(groupToAdd->createDownloadResult());
+      A2_LOG_DEBUG("Commands deleted");
+      groupToAdd->setLastErrorCode(ex.getErrorCode());
+      // We add groupToAdd to e in order to it is processed in
+      // removeStoppedGroup().
+      requestGroups_.push_back(groupToAdd);
+      requestQueueCheck();
     }
+    util::executeHookByOptName
+      (groupToAdd, e->getOption(), PREF_ON_DOWNLOAD_START);
   }
   if(!temp.empty()) {
     reservedGroups_.insert(reservedGroups_.begin(), temp.begin(), temp.end());
   }
   if(count > 0) {
     e->setNoWait(true);
-    if(logger_->debug()) {
-      logger_->debug("%d RequestGroup(s) added.", count);
-    }
+    e->setRefreshInterval(0);
+    A2_LOG_DEBUG(fmt("%d RequestGroup(s) added.", count));
   }
 }
 
@@ -546,7 +552,7 @@ void RequestGroupMan::save()
       try {
         (*itr)->saveControlFile();
       } catch(RecoverableException& e) {
-        logger_->error(EX_EXCEPTION_CAUGHT, e);
+        A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
       }
     }
   }
@@ -565,26 +571,20 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   size_t finished = 0;
   size_t error = removedErrorResult_;
   size_t inprogress = 0;
-  downloadresultcode::RESULT lastError = removedLastErrorResult_;
+  error_code::Value lastError = removedLastErrorResult_;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
       itr != eoi; ++itr) {
-    if((*itr)->result == downloadresultcode::FINISHED) {
+    if((*itr)->belongsTo != 0) {
+      continue;
+    }
+    if((*itr)->result == error_code::FINISHED) {
       ++finished;
     } else {
       ++error;
       lastError = (*itr)->result;
     }
   }
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
-    DownloadResultHandle result = (*itr)->createDownloadResult();
-    if(result->result == downloadresultcode::FINISHED) {
-      ++finished;
-    } else {
-      ++inprogress;
-    }
-  }
   return DownloadStat(finished, error, inprogress, reservedGroups_.size(),
                       lastError);
 }
@@ -615,11 +615,14 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
       itr != eoi; ++itr) {
+    if((*itr)->belongsTo != 0) {
+      continue;
+    }
     std::string status;
-    if((*itr)->result == downloadresultcode::FINISHED) {
+    if((*itr)->result == error_code::FINISHED) {
       status = MARK_OK;
       ++ok;
-    } else if((*itr)->result == downloadresultcode::IN_PROGRESS) {
+    } else if((*itr)->result == error_code::IN_PROGRESS) {
       status = MARK_INPR;
       ++inpr;
     } else {
@@ -628,35 +631,6 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
     }
     o << formatDownloadResult(status, *itr) << "\n";
   }
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
-    DownloadResultHandle result = (*itr)->createDownloadResult();
-    std::string status;
-    if(result->result == downloadresultcode::FINISHED) {
-      status = MARK_OK;
-      ++ok;
-    } else {
-      // Since this RequestGroup is not processed by ProcessStoppedRequestGroup,
-      // its download stop time is not reseted.
-      // Reset download stop time and assign sessionTime here.
-      (*itr)->getDownloadContext()->resetDownloadStopTime();
-      result->sessionTime =
-        (*itr)->getDownloadContext()->calculateSessionTime();
-      status = MARK_INPR;
-      ++inpr;
-    }
-    o << formatDownloadResult(status, result) << "\n";
-  }
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        reservedGroups_.begin(), eoi = reservedGroups_.end();
-      itr != eoi; ++itr) {
-    if(!(*itr)->isPauseRequested()) {
-      continue;
-    }
-    SharedHandle<DownloadResult> result = (*itr)->createDownloadResult();
-    ++inpr;
-    o << formatDownloadResult(MARK_INPR, result) << "\n";
-  }
   if(ok > 0 || err > 0 || inpr > 0) {
     o << "\n"
       << _("Status Legend:") << "\n";
@@ -695,11 +669,12 @@ std::string RequestGroupMan::formatDownloadResult(const std::string& status, con
   return o.str();
 }
 
+namespace {
 template<typename StringInputIterator, typename FileEntryInputIterator>
-static bool sameFilePathExists(StringInputIterator sfirst,
-                               StringInputIterator slast,
-                               FileEntryInputIterator ffirst,
-                               FileEntryInputIterator flast)
+bool sameFilePathExists(StringInputIterator sfirst,
+                        StringInputIterator slast,
+                        FileEntryInputIterator ffirst,
+                        FileEntryInputIterator flast)
 {
   for(; ffirst != flast; ++ffirst) {
     if(std::binary_search(sfirst, slast, (*ffirst)->getPath())) {
@@ -708,6 +683,7 @@ static bool sameFilePathExists(StringInputIterator sfirst,
   }
   return false;
 }
+} // namespace
 
 bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) const
 {
@@ -761,7 +737,7 @@ TransferStat RequestGroupMan::calculateStat()
 }
 
 SharedHandle<DownloadResult>
-RequestGroupMan::findDownloadResult(gid_t gid) const
+RequestGroupMan::findDownloadResult(a2_gid_t gid) const
 {
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator i =
         downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi; ++i) {
@@ -772,6 +748,18 @@ RequestGroupMan::findDownloadResult(gid_t gid) const
   return SharedHandle<DownloadResult>();
 }
 
+bool RequestGroupMan::removeDownloadResult(a2_gid_t gid)
+{
+  for(std::deque<SharedHandle<DownloadResult> >::iterator i =
+        downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi; ++i) {
+    if((*i)->gid == gid) {
+      downloadResults_.erase(i);
+      return true;
+    }
+  }
+  return false;
+}
+
 void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
 {
   if(maxDownloadResult_ == 0) {
@@ -779,14 +767,14 @@ void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
       for(std::deque<SharedHandle<DownloadResult> >::iterator i =
             downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi;
           ++i) {
-        if((*i)->result != downloadresultcode::FINISHED) {
+        if((*i)->belongsTo == 0 && (*i)->result != error_code::FINISHED) {
           removedLastErrorResult_ = (*i)->result;
           ++removedErrorResult_;
         }
       }
       downloadResults_.clear();
     }
-    if(dr->result != downloadresultcode::FINISHED) {
+    if(dr->belongsTo == 0 && dr->result != error_code::FINISHED) {
       removedLastErrorResult_ = dr->result;
       ++removedErrorResult_;
     }
@@ -797,7 +785,7 @@ void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
         downloadResults_.begin()+curSize-maxDownloadResult_+1;
       for(std::deque<SharedHandle<DownloadResult> >::iterator i =
             downloadResults_.begin(); i != last; ++i) {
-        if((*i)->result != downloadresultcode::FINISHED) {
+        if((*i)->belongsTo == 0 && (*i)->result != error_code::FINISHED) {
           removedLastErrorResult_ = (*i)->result;
           ++removedErrorResult_;
         }
@@ -825,7 +813,7 @@ RequestGroupMan::getOrCreateServerStat(const std::string& hostname,
                                        const std::string& protocol)
 {
   SharedHandle<ServerStat> ss = findServerStat(hostname, protocol);
-  if(ss.isNull()) {
+  if(!ss) {
     ss.reset(new ServerStat(hostname, protocol));
     addServerStat(ss);
   }
@@ -841,14 +829,15 @@ bool RequestGroupMan::loadServerStat(const std::string& filename)
 {
   std::ifstream in(filename.c_str(), std::ios::binary);
   if(!in) {
-    logger_->error(MSG_OPENING_READABLE_SERVER_STAT_FILE_FAILED, filename.c_str());
+    A2_LOG_ERROR(fmt(MSG_OPENING_READABLE_SERVER_STAT_FILE_FAILED,
+                     filename.c_str()));
     return false;
   }
   if(serverStatMan_->load(in)) {
-    logger_->notice(MSG_SERVER_STAT_LOADED, filename.c_str());
+    A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_LOADED, filename.c_str()));
     return true;
   } else {
-    logger_->error(MSG_READING_SERVER_STAT_FILE_FAILED, filename.c_str());
+    A2_LOG_ERROR(fmt(MSG_READING_SERVER_STAT_FILE_FAILED, filename.c_str()));
     return false;
   }
 }
@@ -860,20 +849,20 @@ bool RequestGroupMan::saveServerStat(const std::string& filename) const
   {
     std::ofstream out(tempfile.c_str(), std::ios::binary);
     if(!out) {
-      logger_->error(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
-                     filename.c_str());
+      A2_LOG_ERROR(fmt(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
+                       filename.c_str()));
       return false;
     }
     if(!serverStatMan_->save(out)) {
-      logger_->error(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str());
+      A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
       return false;
     }
   }
   if(File(tempfile).renameTo(filename)) {
-    logger_->notice(MSG_SERVER_STAT_SAVED, filename.c_str());
+    A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_SAVED, filename.c_str()));
     return true;
   } else {
-    logger_->error(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str());
+    A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
     return false;
   }
 }
@@ -898,6 +887,11 @@ bool RequestGroupMan::doesOverallUploadSpeedExceed()
 void RequestGroupMan::getUsedHosts
 (std::vector<std::pair<size_t, std::string> >& usedHosts)
 {
+  // vector of triplet which consists of use count, -download speed,
+  // hostname. We want to sort by least used and faster download
+  // speed. We use -download speed so that we can sort them using
+  // operator<().
+  std::vector<Triplet<size_t, int, std::string> > tempHosts;
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
         requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
     const std::deque<SharedHandle<Request> >& inFlightReqs =
@@ -906,22 +900,27 @@ void RequestGroupMan::getUsedHosts
           inFlightReqs.begin(), eoj = inFlightReqs.end(); j != eoj; ++j) {
       uri::UriStruct us;
       if(uri::parse(us, (*j)->getUri())) {
-        std::vector<std::pair<size_t, std::string> >::iterator k;
-        std::vector<std::pair<size_t, std::string> >::iterator eok =
-          usedHosts.end();
-        for(k =  usedHosts.begin(); k != eok; ++k) {
-          if((*k).second == us.host) {
+        std::vector<Triplet<size_t, int, std::string> >::iterator k;
+        std::vector<Triplet<size_t, int, std::string> >::iterator eok =
+          tempHosts.end();
+        for(k =  tempHosts.begin(); k != eok; ++k) {
+          if((*k).third == us.host) {
             ++(*k).first;
             break;
           }
         }
         if(k == eok) {
-          usedHosts.push_back(std::make_pair(1, us.host));
+          SharedHandle<ServerStat> ss = findServerStat(us.host, us.protocol);
+          int invDlSpeed = (ss && ss->isOK()) ?
+            -(static_cast<int>(ss->getDownloadSpeed())) : 0;
+          tempHosts.push_back(makeTriplet(1, invDlSpeed, us.host));
         }
       }
     }
   }
-  std::sort(usedHosts.begin(), usedHosts.end());
+  std::sort(tempHosts.begin(), tempHosts.end());
+  std::transform(tempHosts.begin(), tempHosts.end(),
+                 std::back_inserter(usedHosts), Tuple2Pair<1, 3>());
 }
 
 } // namespace aria2
