@@ -22,8 +22,9 @@ CPPUNIT_TEST_SUITE_REGISTRATION( DefaultDiskWriterTest );
 
 void DefaultDiskWriterTest::testSize()
 {
-  DefaultDiskWriter dw;
-  dw.openExistingFile("4096chunk.txt");
+  DefaultDiskWriter dw(A2_TEST_DIR"/4096chunk.txt");
+  dw.enableReadOnly();
+  dw.openExistingFile();
   CPPUNIT_ASSERT_EQUAL((uint64_t)4096ULL, dw.size());
 }
 
