@@ -1,5 +1,5 @@
-#ifndef _D_MOCK_DHT_MESSAGE_FACTORY_H_
-#define _D_MOCK_DHT_MESSAGE_FACTORY_H_
+#ifndef D_MOCK_DHT_MESSAGE_FACTORY_H
+#define D_MOCK_DHT_MESSAGE_FACTORY_H
 
 #include "DHTMessageFactory.h"
 #include "DHTNode.h"
@@ -81,18 +81,9 @@ public:
   createGetPeersReplyMessage
   (const SharedHandle<DHTNode>& remoteNode,
    const std::vector<SharedHandle<DHTNode> >& closestKNodes,
-                             const std::string& token,
-                             const std::string& transactionID)
-  {
-    return SharedHandle<DHTResponseMessage>();
-  }
-
-
-  virtual SharedHandle<DHTResponseMessage>
-  createGetPeersReplyMessage(const SharedHandle<DHTNode>& remoteNode,
-                             const std::vector<SharedHandle<Peer> >& peers,
-                             const std::string& token,
-                             const std::string& transactionID)
+   const std::vector<SharedHandle<Peer> >& peers,
+   const std::string& token,
+   const std::string& transactionID)
   {
     return SharedHandle<DHTResponseMessage>();
   }
@@ -129,4 +120,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_MOCK_DHT_MESSAGE_FACTORY_H_
+#endif // D_MOCK_DHT_MESSAGE_FACTORY_H
